#!/bin/sh
# Run this to generate all the initial makefiles, etc.

DIE=0
package=libcdx-jpeg
srcfile=libcdx-jpeg/jpeg.h

# needed to find parent specialization scripts and such
PATH=$PATH:`pwd`/../specialib/scripts
export PKG_CONFIG_PATH=$PKG_CONFIG_PATH:`pwd`/../specialib

(specialize --version) < /dev/null > /dev/null 2>&1 || {
	echo
	echo "You must have specialib installed to compile $package."
	echo "Download the appropriate package for your distribution,"
	echo "or get it at http://codecs.org/"
	DIE=1
}

(autoconf --version) < /dev/null > /dev/null 2>&1 || {
	echo
	echo "You must have autoconf installed to compile $package."
	echo "Download the appropriate package for your distribution,"
	echo "or get the source tarball at ftp://ftp.gnu.org/pub/gnu/"
	DIE=1
}

(automake --version) < /dev/null > /dev/null 2>&1 || {
	echo
	echo "You must have automake installed to compile $package."
	echo "Get ftp://ftp.cygnus.com/pub/home/tromey/automake-1.2d.tar.gz"
	echo "(or a newer version if it is available)"
	DIE=1
}

if test "$DIE" -eq 1; then
	exit 1
fi

test -f $srcfile || {
	echo "You must run this script in the top-level $package directory"
	exit 1
}

if test -z "$*"; then
	echo "I am going to run ./configure with no arguments - if you wish "
        echo "to pass any to it, please specify them on the $0 command line."
fi

specialize
specialib_m4dir=`pkg-config specialib --variable=m4dir`
aclocal -I $specialib_m4dir $ACLOCAL_FLAGS || {
	echo "aclocal failed - check that all needed development files are present on system"
	exit 1
}
autoheader
autoconf
automake --add-missing

# now remove the cache, because it can be considered dangerous in this case
rm -f config.cache

./configure --enable-maintainer-mode "$@"

echo 
echo "Now type 'make' to compile $package."
