/* convert block8x8 f64 to s16 functions
 * Copyright (C) 2001,2002,2003  David A. Schleef <ds@schleef.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <specialib/sl_types.h>
#include <math.h>


#define BLOCK8x8_F64(ptr, stride, row, column) \
	(*((double *)((void *)ptr + stride*row) + column))

#define BLOCK8x8_PTR_F64(ptr, stride, row, column) \
	((double *)((void *)ptr + stride*row) + column)

#define BLOCK8x8_S16(ptr, stride, row, column) \
	(*((int16_t *)((void *)ptr + stride*row) + column))

#define BLOCK8x8_U8(ptr, stride, row, column) \
	(*((uint8_t *)((void *)ptr + stride*row) + column))


/**
 * sl_convert8x8_S16_F64:
 * @dst: destination array
 * @dstr: destination row stride
 * @src: source array
 * @sstr: source row stride
 * #arch C
 *
 * Convert an 8x8 F64 block to S16.
 */
SL_FUNC void _sl_convert8x8_S16_F64__c(int16_t *dest, int dstr, double *src,
	int sstr)
{
	int i,j;

	for(i=0;i<8;i++){
		for(j=0;j<8;j++){
			BLOCK8x8_S16(dest,dstr,i,j) =
				rint(BLOCK8x8_F64(src,sstr,i,j));
		}
	}
}

/**
 * sl_convert8x8_F64_S16:
 * @dst: destination array
 * @dstr: destination row stride
 * @src: source array
 * @sstr: source row stride
 * #arch C
 *
 * Convert an 8x8 S16 block to F64.
 */
SL_FUNC void _sl_convert8x8_F64_S16__c(double *dest, int dstr, int16_t *src,
	int sstr)
{
	int i,j;

	for(i=0;i<8;i++){
		for(j=0;j<8;j++){
			BLOCK8x8_F64(dest,dstr,i,j) =
				BLOCK8x8_S16(src,sstr,i,j);
		}
	}
}


/**
 * sl_clipconvert8x8_U8_S16:
 * @dst: destination array
 * @dstr: destination row stride
 * @src: source array
 * @sstr: source row stride
 * #arch C
 *
 * Converts an 8x8 S16 block to U8, clipping values.
 */
SL_FUNC void _sl_clipconvert8x8_U8_S16__c(uint8_t *dest, int dstr, int16_t *src,
	int sstr)
{
	int i,j;
	int16_t x;

	for(i=0;i<8;i++){
		for(j=0;j<8;j++){
			x = BLOCK8x8_S16(src,sstr,i,j);
			if(x<0)x=0;
			if(x>255)x=255;
			BLOCK8x8_U8(dest,dstr,i,j) = x;
		}
	}
}




