#include <libcdx-jpeg/jpeg.h>
#include <bitstream/bitstream.h>
#include <bitstream/vlctable.h>

void cdx_jpeg_parse_SOI(bitstream_t *bs) {
  fprintf(stderr,"'parsing' SOI marker\n");
}

void cdx_jpeg_parse_EOI(bitstream_t *bs) {
}

void cdx_jpeg_parse_quant_tables(bitstream_t *bs,cdx_jpeg_quant_tables_t *tab) {
  int length,used;
  int pq,tq,i;
  short *q;

  length = bitstream_get(bs,16);
  used = 2;
  fprintf(stderr,"parsing quantization tables, total %d bytes\n",length);

  while (used < length) {
    pq = bitstream_get(bs,4);
    tq = bitstream_get(bs,4);

    q = &tab->table[tq][0];
    if (pq) {
      used += 129;
      for (i=0;i<64;i++)
        q[i] = bitstream_get(bs,16);
    } else {
      used += 65;
      for (i=0;i<64;i++)
        q[i] = bitstream_get(bs,8);
    }
  }
}

void cdx_jpeg_parse_baseline_dct_sof(bitstream_t *bs,cdx_jpeg_sof_t *sof) {
  int length;
  int i;
  int max_h_over = 0, max_v_over = 0;
  int rowstride,lines;
  int imagesize;

  length = bitstream_get(bs,16);

  sof->depth = bitstream_get(bs,8);
  sof->height = bitstream_get(bs,16);
  sof->width = bitstream_get(bs,16);
  sof->n_components = bitstream_get(bs,8);

  fprintf(stderr,"have baseline DCT SOF: depth %d, %dx%d pixels, %d components\n",sof->depth,sof->width,sof->height,sof->n_components);

  for (i=0;i<sof->n_components;i++) {
    sof->components[i].id = bitstream_get(bs,8);
    sof->components[i].h_oversample = bitstream_get(bs,4);
    sof->components[i].v_oversample = bitstream_get(bs,4);
    sof->components[i].quant_table = bitstream_get(bs,8);
    fprintf(stderr,"component %d: id %d, hxv oversample %dx%d, quant_table %d\n",i,
sof->components[i].id,sof->components[i].h_oversample,sof->components[i].v_oversample,sof->components[i].quant_table);

    if (max_h_over < sof->components[i].h_oversample) max_h_over = sof->components[i].h_oversample;
    if (max_v_over < sof->components[i].v_oversample) max_v_over = sof->components[i].v_oversample;
  }

  sof->width_blocks = (sof->width + 8*max_h_over - 1) / (8*max_h_over);
  sof->height_blocks = (sof->height + 8*max_v_over - 1) / (8*max_v_over);

  for (i=0;i<sof->n_components;i++) {
    sof->components[i].h_subsample = max_h_over / sof->components[i].h_oversample;
    sof->components[i].v_subsample = max_v_over / sof->components[i].v_oversample;

    rowstride = sof->width_blocks * 8 * max_h_over / sof->components[i].h_subsample;
    lines = sof->height_blocks * 8 * max_v_over / sof->components[i].v_subsample;
    imagesize = rowstride * lines;
    sof->components[i].rowstride = rowstride;
  }
}

void cdx_jpeg_parse_huffman_tables(bitstream_t *bs,cdx_jpeg_huffman_tables_t *tabs) {
  int tc,th;
  int length,used;
  vlctable_t *tab;
  int huffsize[16], total;
  int i,j,k;
  uint32_t symbol;
  uint32_t value;

  length = bitstream_get(bs,16);
  used = 2;
  fprintf(stderr,"parsing huffman tables, total %d bytes\n",length);

  while (used < length) {
fprintf(stderr,"starting to read table at bitoffset %d\n",bitstream_get_bitoffset(bs));

    tc = bitstream_get(bs,4);
    th = bitstream_get(bs,4);
    fprintf(stderr,"tc is %d, th is %d\n",tc,th);

    // determine the number of codewords
    total = 0;
    for (i=0;i<16;i++)
      total += huffsize[i] = bitstream_get(bs,8);

    // create the table
    tab = vlctable_new(16);

    // now walk through all the codes and set up the table
    symbol = 0;
    k = 0;
    for (i=0;i<16;i++) {
      for (j=0;j<huffsize[i];j++) {
        value = bitstream_get(bs,8);
        fprintf(stderr,"adding code %s (%d) = %02x\n",_vlctable_printbits(symbol,i+1),i+1,value);
        vlctable_add_code(tab,symbol,i+1,value);
        symbol++;
        k++;
      }
      if (symbol >= (1<<(i+1))) {
        fprintf(stderr,"bad huffsize[] array!\n");
        return;
      }
      symbol <<= 1;
    }

    // if a vlctable is already in that spot, free it
    if (tc) {
      fprintf(stderr,"adding ac_huff_table %d\n",th);
      if (tabs->ac_huff_tables[th]) vlctable_destroy(tabs->ac_huff_tables[th]);
      tabs->ac_huff_tables[th] = tab;
    } else {
      fprintf(stderr,"adding dc_huff_table %d\n",th);
      if (tabs->dc_huff_tables[th]) vlctable_destroy(tabs->dc_huff_tables[th]);
      tabs->dc_huff_tables[th] = tab;
    }

//    vlctable_show(tab);

    used += 17 + total;
  }

}

static inline _cdx_jpeg_find_component(cdx_jpeg_sof_t *sof, int id) {
  int i;
  for (i=0;i<4;i++) {
    if (sof->components[i].id == id)
      return i;
  }
  return -1;
}

void cdx_jpeg_parse_sos(bitstream_t *bs, cdx_jpeg_decoder_t *dec) {
  cdx_jpeg_sof_t *sof = &(dec->sof);
  int length;
  int n_components;
  int i;
  int component_id, dc_table, ac_table, index;
  int x,y,scan;

  int spectral_start, spectral_end, approx_high, approx_low;

  length = bitstream_get(bs,16);
  fprintf(stderr,"parsing SOS, total %d bytes\n",length);

  n_components = bitstream_get(bs,8);
  scan = 0;
  for (i=0;i<n_components;i++) {
    component_id = bitstream_get(bs,8);
    dc_table = bitstream_get(bs,4);
    ac_table = bitstream_get(bs,4);
    index = _cdx_jpeg_find_component(sof,component_id);

    fprintf(stderr,"component #%d, id %d, index #%d, dc_table %d, ac_table %d\n",
            i,component_id,index,dc_table,ac_table);

    for (y=0;y<sof->components[index].v_oversample;y++) {
      for (x=0;x<sof->components[index].h_oversample;x++) {
        dec->scanlist[scan].component_id = index;
        dec->scanlist[scan].dc_table = dc_table;
        dec->scanlist[scan].ac_table = ac_table;
        dec->scanlist[scan].quant_table = sof->components[index].quant_table;
        dec->scanlist[scan].x = x;
        dec->scanlist[scan].y = y;
        dec->scanlist[scan].offset = y*8*sof->components[index].rowstride + x*8;
        fprintf(stderr,"scan %d (comp %d, v %d, h %d) at offset %d\n",scan,i,y,x,dec->scanlist[scan].offset);
        scan++;
      }
    }

    if (dec->scan_h_subsample < sof->components[index].h_oversample)
      dec->scan_h_subsample = sof->components[index].h_oversample;
    if (dec->scan_v_subsample < sof->components[index].v_oversample)
      dec->scan_v_subsample = sof->components[index].v_oversample;
  }

  dec->numscanlists = scan;

  spectral_start = bitstream_get(bs,8);
  spectral_end = bitstream_get(bs,8);
  fprintf(stderr,"spectral range [%d,%d]\n",spectral_start,spectral_end);
  approx_high = bitstream_get(bs,4);
  approx_low = bitstream_get(bs,4);
  fprintf(stderr,"approx range [%d,%d]\n",approx_low,approx_high);
}

int cdx_jpeg_parse_macroblock(bitstream_t *bs, vlctable_t *dc_table, vlctable_t *ac_table, int16_t *block) {
  uint32_t value;
  int k;
  int r,s;
  uint32_t baseoffset = bitstream_get_bitoffset(bs);

  fprintf(stderr,"initial macroblock offset is %d:%d, next 32 bits are %s\n",baseoffset/8,baseoffset%8,_vlctable_printbits(bitstream_show(bs,32),32));

  memset(block,0,sizeof(*block)*64);

  s = vlctable_decode(bs,dc_table);
  value = bitstream_get(bs,s);
  fprintf(stderr,"got %d bits of value: %s\n",s,_vlctable_printbits(value,s));
  fprintf(stderr,"s=%d, block[0]=%d\n",s,value);
  block[0] = value;

  for (k=1;k<64;k++) {
    fprintf(stderr,"\n");
    value = vlctable_decode(bs,ac_table);
    r = value >> 4;
    s = value & 0xf;

    fprintf(stderr,"%d: coeff %d: (%02x) r=%d s=%d ",bitstream_get_bitoffset(bs)-baseoffset,k,value,r,s);

    if (s == 0) {
      if (r == 15) {
        k += 15;
        fprintf(stderr,"skip 15\n");
      } else {
        fprintf(stderr,"eob\n");
        break;
      }
    } else {
      k += r;
      if (k >= 64) {
        fprintf(stderr,"overrun in macroblock\n");
        return -1;
      }
      value = bitstream_get(bs,s);
      fprintf(stderr,"got %d bits of value: %s ",s,_vlctable_printbits(value,s));
      if ((value>>(s-1)) == 0)
        value -= (1<<s) - 1;
      block[k] = value;
      fprintf(stderr,"-> %d\n",value);
    }
  }

  for (k=0;k<64;k++) {
    fprintf(stderr,"%-6d ",block[k]);
    if (k%8 == 7) fprintf(stderr,"\n");
  }

  return 0;
}

void cdx_jpeg_parse_entropy_segment(bitstream_t *bs,cdx_jpeg_decoder_t *dec) {
  unsigned int n = 0;
  vlctable_t **dc_tables = dec->huffman.dc_huff_tables;
  vlctable_t **ac_tables = dec->huffman.ac_huff_tables;

  cdx_jpeg_scan_t *scan;
  int dc_table, ac_table;

  int16_t block[64];
  int i, x=0,y=0;

  if (n == 0) n = -1;

  while (--n) {
    for (i=0;i<dec->numscanlists;i++) {
      scan = &(dec->scanlist[i]);
      fprintf(stderr,"%d,%d: component=%d, dc_table=%d, ac_table=%d\n",
              x,y,scan->component_id, scan->dc_table, scan->ac_table);
      cdx_jpeg_parse_macroblock(bs, dc_tables[scan->dc_table], ac_tables[scan->ac_table], block);

      // ...
    }

    x += 8;
    if (x * dec->scan_h_subsample >= dec->sof.width) {
      x = 0;
      y += 8;
    }
    if (y * dec->scan_v_subsample >= dec->sof.height) {
      ;
    }
  }
}


void cdx_jpeg_parse_application0(bitstream_t *bs) {
  int length;
  int i;

  length = bitstream_get(bs,16);

  for (i=0;i<length-2;i++)
    bitstream_flush(bs,8);
 }

cdx_jpeg_decoder_t *cdx_jpeg_decoder_new() {
  return (cdx_jpeg_decoder_t *)malloc(sizeof(cdx_jpeg_decoder_t));
}

int cdx_jpeg_decode_chunk(cdx_jpeg_decoder_t *dec, bitstream_t *bs) {
  int i,j,n;
  uint32_t marker;
  uint32_t length;

  for (i=0;i<10;i++) {
    fprintf(stderr,"starting to hunt for marker, bitstream offset is %d %% 8 = %d\n",bitstream_get_bitoffset(bs),bitstream_get_bitoffset(bs)%8);

    marker = bitstream_get(bs,8);
    if (marker != 0xff) {
      n = 0;
      while (marker != 0xff)
        marker = bitstream_get(bs,8),n++;
//      fprintf(stderr,"lost sync, skipped %d bytes!\n",n);
    }

    while (marker == 0xff)
      marker = bitstream_get(bs,8);

    fprintf(stderr,"startcode is 0x%02x\n",marker);

    switch (marker) {
      case JPEG_MARKER_SOI:
        cdx_jpeg_parse_SOI(bs);
        break;
      case JPEG_MARKER_DQT:
        cdx_jpeg_parse_quant_tables(bs,&dec->quant);
        break;
      case JPEG_MARKER_DHT:
        cdx_jpeg_parse_huffman_tables(bs,&dec->huffman);
        break;
      case JPEG_MARKER_SOF_0:
        cdx_jpeg_parse_baseline_dct_sof(bs,&dec->sof);
        break;
      case JPEG_MARKER_SOS:
        cdx_jpeg_parse_sos(bs,dec);
        cdx_jpeg_parse_entropy_segment(bs,dec);
        break;
      case 0xe1:
        cdx_jpeg_parse_application0(bs);
        break;
      default:
//        length = bitstream_get(bs,16);
//        fprintf(stderr,"there are %d bytes following this startcode\n");
//        for (j=0;j<length;j++) bitstream_flush(bs,8);
    }
  }
}
